from langchain.prompts import ChatPromptTemplate
from langchain_core.output_parsers import StrOutputParser
from langchain_core.runnables import RunnablePassthrough
from models import OllamaChatModel
from embeddings import similarity_search
from text_processing import remove_unwanted_content
from langchain_openai import OpenAIEmbeddings
from sklearn.metrics.pairwise import cosine_similarity
import os



taide_llm = OllamaChatModel(model_name="taide-local-3")

def get_context(query, index, docs):
    results = similarity_search(query, index, docs)
    context = "\n".join([doc.page_content for doc, _ in results])

    # 印出問題和搜尋到的文檔的前幾個字
    print(f"Question: {query}")
    print("Retrieved documents:")
    for i, (doc, similarity) in enumerate(results):
        print(f"Doc {i+1} (similarity: {similarity:.4f}): {doc.page_content[:50]}...")
    print("-" * 50)
    
    return context, results[0][1] if results else (context, 0)  # Return context and top similarity score

def simple_rag_prompt(retrieval_chain, question):
    template = """Answer the following question based on this context:

    {context}

    Question: {question}
    Output in user's language. If the question is in zh-tw, then the output will be in zh-tw. If the question is in English, then the output will be in English.
    You should not mention anything about "根據提供的文件內容" or other similar terms. Do not mention anything relate with the Documents or context.
    If you don't know the answer, just say that "很抱歉，目前我無法回答您的問題，請將您的詢問發送至 test@email.com 以便獲得更進一步的幫助，謝謝。I'm sorry I cannot answer your question. Please send your question to test@email.com for further assistance. Thank you."
    """

    prompt = ChatPromptTemplate.from_template(template)
    context, similarity_score = retrieval_chain(question)

    final_rag_chain = (
        {"context": lambda x: context, 
        "question": lambda x: x} 
        | prompt
        | taide_llm
        | StrOutputParser()
    )

    try:
        answer = final_rag_chain.invoke(question)
        answer = remove_unwanted_content(answer)  # 使用 remove_unwanted_content
        return answer, similarity_score
    except Exception as e:
        print(f"Error invoking rag_chain: {e}")
        return "Error occurred while processing the question.", 0

def calculate_similarity(text1, text2):
    embeddings = OpenAIEmbeddings(openai_api_key=os.getenv("OPENAI_API_KEY"))
    emb1 = embeddings.embed_query(text1)
    emb2 = embeddings.embed_query(text2)
    return cosine_similarity([emb1], [emb2])[0][0]